/***************************************************************************
 *                                                                         *
 *  MAXIMUS-CBCS Source Code, Version 1.02                                 *
 *  Copyright 1989, 1990 by Scott J. Dudley.  All rights reserved.         *
 *                                                                         *
 *  Chat module.  IPCxx.BBS writing and checking routines, and             *
 *  resident portion of inbound message handler.                           *
 *                                                                         *
 *  For complete details of the licensing restrictions, please refer to    *
 *  the licence agreement, which is published in its entirety in MAX.C     *
 *  and LICENCE.MAX.                                                       *
 *                                                                         *
 *  USE OF THIS FILE IS SUBJECT TO THE RESTRICTIONS CONTAINED IN THE       *
 *  MAXIMUS-CBCS LICENSING AGREEMENT.  IF YOU DO NOT FIND THE TEXT OF THIS *
 *  AGREEMENT IN ANY OF THE  AFOREMENTIONED FILES, OR IF YOU DO NOT HAVE   *
 *  THESE FILES, YOU SHOULD IMMEDIATELY CONTACT THE AUTHOR AT ONE OF THE   *
 *  ADDRESSES LISTED BELOW.  IN NO EVENT SHOULD YOU PROCEED TO USE THIS    *
 *  FILE WITHOUT HAVING ACCEPTED THE TERMS OF THE MAXIMUS-CBCS LICENSING   *
 *  AGREEMENT, OR SUCH OTHER AGREEMENT AS YOU ARE ABLE TO REACH WITH THE   *
 *  AUTHOR.                                                                *
 *                                                                         *
 *  You can contact the author at one of the address listed below:         *
 *                                                                         *
 *  Scott Dudley           FidoNet  1:249/106                              *
 *  777 Downing St.        IMEXnet  89:483/202                             *
 *  Kingston, Ont.         Internet f106.n249.z1.fidonet.org               *
 *  Canada - K7M 5N3       BBS      (613) 389-8315 - HST/14.4K, 24hrs      *
 *                                                                         *
 ***************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <io.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <mem.h>
#include <string.h>
#include "prog.h"
#include "max.h"
#include "alc.h"

/* MSC replacement for the TC file locking functions. */

#ifdef __MSC__
#include <sys/locking.h>

int lock(int fh, long offset, long len)
{
    if(-1L != lseek(fh, offset, SEEK_SET))
        return locking(fh, LK_NBLCK, len);
    else
        return(-1);
}

int unlock(int fh, long offset, long len)
{
    if(-1L != lseek(fh, offset, SEEK_SET))
        return locking(fh, LK_UNLCK, len);
    else
        return(-1);
}

#endif  /* __MSC__ */









struct _cstat cstat;

int cur_ch=-1,
    num_task=0,
    *tasks;

long scan_time=NORM_SCAN_TIME;



void pascal ChatSetStatus(int avail,char *status)
{
  char temp[PATHLEN];

  int flag,
      fd;

  if (! *PRM(ipc_path))
    return;

  sprintf(temp,ipcxx_bbs,PRM(ipc_path),task_num);

  if (fexist(temp))
  {
    if ((fd=ChatOpenIPC(task_num))==-1)
      return;

    flag=TRUE;
  }
  else
  {
    if ((fd=sopen(temp,O_WRONLY | O_CREAT | O_TRUNC | O_BINARY,SH_DENYNONE,S_IREAD | S_IWRITE))==-1)
      return;

    flag=FALSE;
  }

  if (read(fd,(char *)&cstat,sizeof(struct _cstat)) < sizeof(struct _cstat))
    memset(&cstat,'\0',sizeof(struct _cstat));

  cstat.avail=avail;
  strcpy(cstat.username,usr.name);
  strcpy(cstat.status,status);

  if (cstat.msgs_waiting==0)
    cstat.msg_offset=(long)sizeof(struct _cstat);

  lseek(fd,0L,SEEK_SET);
  write(fd,(char *)&cstat,sizeof(struct _cstat));

  if (flag)
    ChatCloseIPC(fd);
  else close(fd);
}




/* These two functions save and restore the current CHAT availability and   *
 * status.                                                                  */

void pascal ChatSaveStatus(struct _css *css)
{
  if (! *PRM(ipc_path))
    return;

  css->avail=cstat.avail;
  strcpy(css->status,cstat.status);
}



void pascal ChatRestoreStatus(struct _css *css)
{
  if (! *PRM(ipc_path))
    return;

  ChatSetStatus(css->avail,css->status);
}





int pascal ChatSendMsg(int tid,int type,int len,char *msg)
{
  struct _cdat cdat;
  struct _cstat cs;

  int fd;

  if ((fd=ChatOpenIPC(tid))==-1)
    return -1;

  read(fd,(char *)&cs,sizeof(struct _cstat));

  cs.msgs_waiting++;

  cdat.tid=task_num;
  cdat.type=type;
  cdat.len=len;
  cdat.prior_offset=cs.msg_offset;
  cdat.prior_len=cs.msglen;

  /* Seek to the end of this message */
  lseek(fd,cs.msg_offset+(long)sizeof(struct _cdat)+(long)cs.msglen,SEEK_SET);

  cs.msg_offset=tell(fd);
  cs.msglen=len;

  write(fd,(char *)&cdat,sizeof(struct _cdat));
  write(fd,msg,len);

  lseek(fd,0L,SEEK_SET);
  write(fd,(char *)&cs,sizeof(struct _cstat));

  ChatCloseIPC(fd);

  return 0;
}




int pascal ChatGetMsg(int *tid,int *type,int *len,char *msg,int maxlen)
{
  struct _cstat cs;
  struct _cdat cdat;

  int fd;

  if ((fd=ChatOpenIPC(task_num))==-1)
    return -1;

  read(fd,(char *)&cs,sizeof(struct _cstat));

  if (! cs.msgs_waiting)
  {
    ChatCloseIPC(fd);
    return 1;
  }

  cs.msgs_waiting--;

  lseek(fd,cs.msg_offset,SEEK_SET);
  read(fd,(char *)&cdat,sizeof(struct _cdat));

  *tid=cdat.tid;
  *type=cdat.type;
  *len=cdat.len;

  read(fd,msg,min(maxlen,cdat.len));

  /* Now update the offset */

  cs.msg_offset=cdat.prior_offset;
  cs.msglen=cdat.prior_len;

  lseek(fd,0L,SEEK_SET);
  write(fd,(char *)&cs,sizeof(struct _cstat));

  ChatCloseIPC(fd);

  return 0;
}



int pascal ChatOpenIPC(int tid)
{
  char temp[PATHLEN];

  int tries,
      fd;

  if (! *PRM(ipc_path))
  {
    #ifdef DEBUG
    Printf("IPC:nopath ");
    #endif

    return -1;
  }

  sprintf(temp,ipcxx_bbs,PRM(ipc_path),tid);

  if ((fd=shopen(temp,O_RDWR | O_BINARY))==-1)
  {
    #ifdef DEBUG
    Printf("IPC:openerr ");
    #endif

    return -1;
  }

  if (! (prm.flags2 & FLAG2_noshare))
  {
    for (tries=0;tries < MAX_TRIES;tries++)
    {
      if (lock(fd,0L,1L)==-1)
      {
        Giveaway_Slice();
        continue;
      }
      else break;
    }

    if (tries==MAX_TRIES)   /* Blocked out */
    {
      #ifdef DEBUG
      Printf("IPC:max ");
      #endif

      close(fd);
      return -1;
    }
  }

  lseek(fd,0L,SEEK_SET);
  return fd;
}



void pascal ChatCloseIPC(int fd)
{
  if (! *PRM(ipc_path))
    return;

  if (! (prm.flags2 & FLAG2_noshare))
    unlock(fd,0L,1L);
 
  close(fd);
}



void pascal Check_For_Message(char *s1,char *s2)
{
  int redo,
      tid,
      type,
      len,
      tl;

  char msg[PATHLEN];

  static long last_check=0L;

  tl=FALSE;

  if (*PRM(ipc_path) && usernum != -1 && task_num &&
      time(NULL) >= last_check+scan_time)
  {
    last_check=time(NULL);

    redo=FALSE;

    while (ChatGetMsg(&tid,&type,&len,msg,PATHLEN)==0)
      ChatHandleMessage(tid,type,len,msg,&tl,&redo);

    if (tl)
      TopLine(LINE_top);

    if (redo)
    {
      if (s1 && s2)
        Printf(ss,s1,s2);

      vbuf_flush();
    }
  }
}


